% Kepler's perturbed 2 body problem.

% Copyright 2005-2005 Patrick Meury
% SAM - Seminar for Applied Mathematics
% ETH-Zentrum
% CH-8092 Zurich, Switzerland

  h = 0.01;                               % Steps size
  nSteps = 10000;                         % Time steps
  tol = 1e-8;                             % Error tolerance
  maxit = 20;                             % Maximum iterations
  e = 0.6;                                % Eccentricity
  mu = 0.005;                             % Relativistic parameter
  y0 = [1-e; 0; 0; sqrt(1+e)/sqrt(1-e)];  % Initial value
  f_handle = @f_pkepler;                  % Right hand side
  g_handle = @g_pkepler;                  % Constraints

  % Compute total energy and angular momentum
  
  g = g_handle(y0,mu,0,0);
  H0 = g(1);
  L0 = g(2);
  
  % Run Euler schemes
  
  y = impl_mdpt(y0,h,nSteps,tol,maxit,f_handle,mu);
  
  % Compute values of constraints
  
  g = g_handle(y,mu,H0,L0);
  
  % Generate figures
  
  figure('Name','Perturbed Kepler problem');
  plot(y(1,:),y(2,:),'r-');
  legend('impl. midpoint','Location','SouthEast');
  title('{\bf Perturbed Kepler problem}');
  xlabel('{\bf q_1}');
  ylabel('{\bf q_2}');
  set(gca,'XLim',[-2 2],'YLim',[-2 2],'DataAspectRatio',[1 1 1]);
  
  figure('Name','Constraints');
  plot(0:1:nSteps,sqrt(sum(g.^2,1)),'r.','MarkerSize',5);
  title('{\bf l^2-norm of constraints}');
  xlabel('{\bf Step number}');
  ylabel('{\bf Constraints [log]}');
  legend('impl. midpoint','Location','SouthEast');
  set(gca,'YScale','log');
 
  % Clear memory
  
  clear all;
  