% Mathematical pendulum.

% Copyright 2005-2005 Patrick Meury
% SAM - Seminar for Applied Mathematics
% ETH-Zentrum
% CH-8092 Zurich, Switzerland

  h = 0.005;                  % Steps size
  nSteps = 10000;             % Time steps
  tol = 1e-8;                 % Error tolerance
  maxit = 20;                 % Maximum iterations
  y0 = [1; 1; 0; 0]/sqrt(2);  % Initial value
  f_handle = @f_mp;           % Right hand side
  g_handle = @g_mp;           % Constraints
  
  % Compute pendulum length
  
  g = sqrt(g_handle(y0,0,0));
  R = g(1);
  
  % Run Runge-Kutta methods
  
  [B,C] = init_RK4();
  y1 = expl_RKM(y0,h,nSteps,B,C,f_handle);
  y2 = expl_RKM_tsp(y0,h,nSteps,B,C,tol,maxit,f_handle,g_handle,R);
  y3 = expl_RKM_tsp(y0,h,nSteps,B,C,tol,maxit,f_handle,g_handle,R,0);
  
  % Compute values of constraints
  
  g1 = g_handle(y1,R,0);
  g2 = g_handle(y2,R,0);
  g3 = g_handle(y3,R,0);
  
  % Generate figures
  
  figure('Name','Pendulum equation');
  plot(y1(1,:),y1(2,:),'r-');
  legend('RKM4','Location','SouthEast');
  title('{\bf Pendulum equation}');
  xlabel('{\bf q_1}');
  ylabel('{\bf q_2}');
  set(gca,'XLim',[-1.5 1.5],'YLim',[-1.5 1.5],'DataAspectRatio',[1 1 1]);
 
  figure('Name','Pendulum equation');
  plot(y2(1,:),y2(2,:),'b-');
  legend('RKM4 tsp','Location','SouthEast');
  title('{\bf Pendulum equation}');
  xlabel('{\bf q_1}');
  ylabel('{\bf q_2}');
  set(gca,'XLim',[-1.5 1.5],'YLim',[-1.5 1.5],'DataAspectRatio',[1 1 1]);
  
  figure('Name','Pendulum equation');
  plot(y3(1,:),y3(2,:),'g-');
  legend('RKM4 tsp','Location','SouthEast');
  title('{\bf Pendulum equation}');
  xlabel('{\bf q_1}');
  ylabel('{\bf q_2}');
  set(gca,'XLim',[-1.5 1.5],'YLim',[-1.5 1.5],'DataAspectRatio',[1 1 1]);
  
  figure('Name','Constraints');
  plot(0:1:nSteps,sqrt(sum(g1.^2,1)),'r.', ...
       0:1:nSteps,sqrt(sum(g2.^2,1)),'b.', ...
       0:1:nSteps,sqrt(sum(g3.^2,1)),'g.', ...
       'MarkerSize',5);
  legend('RKM4','RKM4 tsp','RKM4 tsp', ...
         'Location','SouthEast');
  title('{\bf l^2-norm of constraint}');
  xlabel('{\bf Step number}');
  ylabel('{\bf Constraint [log]}');
  set(gca,'YScale','log');
 
  % Clear memory
  
  clear all;
  